// config.c

#include "config.h"
#include "charset.h"

#define CONFIG_LINE_LEN INTCOM_LINE_LEN
typedef
struct _config_name_pair
{
  cid config_id;
  char *config_name;
} config_name_pair;

static config_name_pair config_names[] =
/* dava do vztahu cid a jmeno configu */
{
  {  C_CHARSET_SAVED_DEFAULT,       "charset saved default" },
  {  C_CHARSET_META_CAN_OVERRIDE,   "charset meta can override" },
  {  C_CHARSET_NETWORK_OUTPUT,      "charset network output" },
  {  C_CHARSET_NETWORK_INPUT,       "charset network input" },
  {  C_CHARSET_ADD_TO_CONTENT_TYPE, "charset add to content type" },
  {  C_7BIT_ASSUME_ASCII,           "assume ascii" },
  {  C_ROOT_DIR,                    "root dir" },
  {  C_DIR_DELIMITER,               "dir delimiter" },
  {  C_ROOT_URL,                    "root url" },
  {  C_YAAHO_CGI_URL,               "yaaho cgi url" },
  {  C_HTTP_METHOD_POST,            "http method post" },
  {  C_REQUEST_OFFSET,              "request offset" },
  {  C_REQUEST_COUNT_LIMIT,         "request count limit" },
  {  C_LAYOUT_ERROR_PAGE,           "layout error page" },
  {  C_LAYOUT_ANSWER_PAGE_HEADER,   "layout answer page header" },
  {  C_LAYOUT_ANSWER_PAGE_ITEM,     "layout answer page item" },
  {  C_LAYOUT_ANSWER_PAGE_FOOTER,   "layout answer page footer" },
  {  C_PLOTTER_PATHNAME,            "plotter pathname" },
  {  C_SEARCHER_PATHNAME,           "searcher pathname" },
  {  C_FIND_COMMAND,                "find command" },
  { C_NO_CID,                     0 }

};

config_storage config_store;
/* globalni sklad konfigurace */

void
set_config_int(cid CID, int DATA)
{
  config_store.data[ cid_to_index(CID) ].i = (DATA);
};
void
set_config_str(cid CID, char *DATA)
{
  config_store.data[ cid_to_index(CID) ].str = (DATA);
};

#include <stdlib.h> /*malloc*/
#include "cross.h"  /*error codes*/
int
init_config(config_info *info)
/* inicializuje globalni sklad konfigurace, vraci 0 nebo chybu */
{
  int i,cnt;
  for(cnt=0; info[cnt].config_type != C_END; cnt++) ;
  config_store.info = info;
  config_store.data = (config_data*) malloc(cnt*sizeof(config_data));
  if (config_store.data == NULL)
    return OUT_OF_MEMORY;
  /* inicializace obsahu configu */
  for(i=0; i<cnt; i++)
    switch (config_store.info[i].config_type)
     {
      case C_INT:
      case C_BOOL:
        config_store.data[i].i = 0;
        break;
      case C_CHARSET:
        config_store.data[i].i = CH_UNKNOWN;
        break;
   /* FIXME nevim proc mi na dalsi nechce chodit */
      case C_STRING:
      case C_INCLUDE:
        config_store.data[i].str = NULL;
        break;
     }
  return 0;
}
#include "strtools.h"
cid
config_name_to_cid(char *name)
{
  config_name_pair *i;
  int f,num;
  name=skipwhite(name);
  f=sscanf(name, "%d", &num);
  if (f)
    return num;
  /* pokud jsme jako jmeno konfigu dostali hned cislo, verme, ze je to cid */
  for(i=config_names;  i->config_id != C_NO_CID; i++)
    if (i->config_name != NULL)
      if ( 0 == strcmpiw(i->config_name, name) )
        return i->config_id;
  return C_NO_CID;
}
int
cid_to_index(cid c)
/* prevede cid na index v globalnim skladu config_store, -1 pokud neni */
{
  int i;
  for(i=0; config_store.info[i].config_type != C_END; i++)
    if ( config_store.info[i].config_id == c )
      return i;
  return -1;
}


#include <stdio.h>
#include <string.h>

int
parse_config_line(FILE *stream, char *line)
{
  char *nam, *val;
  cid c;
  int i;
  int f, num;
  int ch;
  char *str;
  char *buff, *newbuff;
  int bufflen, linelen;

  nam=skipwhite(line);
  if ( *nam == '#' || *nam == 0)
    /* comment or blank line */
    return 0;
  /* split the line */
  val=strchr(nam,'=');
  *val=0; val++;
  val=skipwhite(val);
  /* nam...name  val...value */
  
  c=config_name_to_cid(nam);
  if (c == C_NO_CID)
    return MISCONF_UNKNOWN_CONFIG_NAME;
  i=cid_to_index(c);
  if (i == -1)
    return MISCONF_UNSUPPORTED_CONFIG_NAME;

  switch ( config_store.info[i].config_type )
  {
    case C_INT:
      f=sscanf(val,"%d",&num);
      if (!f) return MISCONF_INTEGER_EXPECTED;
      config_store.data[i].i=num;
      break;
    case C_BOOL:
      f=sscanf(val,"%d",&num);
      if (!f) return MISCONF_BOOLEAN_EXPECTED;
      config_store.data[i].i=( num != 0 );
      break;
    case C_CHARSET:
      ch=a_to_charset(val);
      if (ch == CH_UNKNOWN) return MISCONF_UNKNOWN_CHARSET;
      config_store.data[i].i=ch;
      break;
    case C_STRING:
      str=strdup(val);
      if ( str == NULL ) return OUT_OF_MEMORY;
      config_store.data[i].str=str;
      break;
    case C_INCLUDE:
      if ( *val=='<' && *(val+1)=='<' )
       {
        val+=2; val=skipwhite(val);
        if (*val == 0)
          return MISCONF_INCLUDE_STOP_UNSPECIFIED;
        /* val je ted znacka konce includu */
        /* val je ale pointer do bufferu line, ktery brzo prepiseme
           udelejme si kopii!*/
        val=strdup(val);
        if (val ==NULL) return OUT_OF_MEMORY;
        buff=(char*) malloc(1);
        if (buff==NULL) return OUT_OF_MEMORY;
        *buff=0;
        bufflen=1;
        while ( NULL != ( line = my_fgets(line, CONFIG_LINE_LEN, stream)) )
                 /* neni konec souboru */
         {
          if (0==strcmp(val,line))
            /* jsme na konci */
            break;
          linelen=strlen(line);
          /* nutno realokovat */
          newbuff=(char*)realloc(buff, bufflen+linelen+1 );
          if (newbuff==NULL)
            /* free(buff); */
            return OUT_OF_MEMORY;
          buff=newbuff;
          strcpy(buff+bufflen-1, line);
          bufflen+=linelen+1;
          *(buff+bufflen-2)='\n'; *(buff+bufflen-1)=0;
         } /*while*/
        if (line==NULL)
          /* necetl dal, protoze narazil na konec souboru
             spravne mel ale narazit na koncovy retezec !! */
          return MISCONF_UNEXPECTED_END_OF_FILE;
        free(val);
        config_store.data[i].str=buff;
        break;
       }
      else
       {
        /*FIXME*/
        return MISCONF_EXTERNAL_INCLUDES_UNSUPPORTED;
       }
    default: /*way to read config according to config type*/
      return MISCONF_INTERNAL_ERROR;
  }
  return 0;
}

int
read_config(FILE *input)
/* nacist konfiguraci z daneho proudu, zastavit se ale po CONFIG_FINISHED */
{
  char line[CONFIG_LINE_LEN];
  int err;
  while ( my_fgets(line,CONFIG_LINE_LEN,input) != NULL )
   {
    if (strcmp(line,CONFIG_FINISHED) == 0)
      break;
    else
      if (0 != ( err = parse_config_line(input,line) ) )
        return err;
   }
  
  return 0;
}

int
read_config_from_file(char *fname)
{
  FILE *f;
  int err;
  /* **** Nacist konfiguraci ze souboru */
  f=fopen(fname, "rt");
  if (f == NULL)
    return CANT_READ_CONFIG;
  err=read_config(f);
  fclose(f);
  return err;
}
int
read_config_from_file_and_stdin(char *fname)
{
  int err;
  err=read_config_from_file(fname);
  if (err)
    return err;
  /* **** Nacist zbytek konfigurace ze stdin */
  err=read_config(stdin);
  return err;
}


void
flush_config(FILE *output)
/* vyplivnout exportovane cleny konfigurace, pak CONFIG_FINISHED */
{
  int i;
  char *str;
  char *chname;
  for(i=0; config_store.info[i].config_type != C_END; i++)
   {
    if (config_store.info[i].config_exported != C_PASS)
      continue; /* neexportovat vse, ale jen exportovane */
    switch ( config_store.info[i].config_type )
     {
      case C_INT:
      case C_BOOL:
        fprintf(output, "%d=", config_store.info[i].config_id);
        fprintf(output, "%d\n", config_store.data[i].i);
        break;
      case C_CHARSET:
        chname=charset_to_a(config_store.data[i].i,charset_name_MIME);
        if (chname)
         {
          fprintf(output, "%d=", config_store.info[i].config_id);
          fprintf(output, "%s\n", chname);
         }
        break;
      case C_STRING:
        str=config_store.data[i].str;
        if (str)
         {
          fprintf(output, "%d=", config_store.info[i].config_id);
          fprintf(output, "%s\n", str);
         }
        break;
      default:
        /* FIXME */
        /* sem se vubec nema dostat!
           zbyva jeste C_INCLUDE, ale ten se z definice neexportuje
        */
        break;
     } /*switch*/
   }
  fprintf(output,"%s\n",CONFIG_FINISHED);
  return;
}

