/* cross.c data spolecna pro vsechny castmi yaaho
*/
#include "cross.h"
/* --- interni komunikace --- */
int intcom_item_order[]=
{
  I_PATHNAME,
  I_STAT,
  I_CHARSET,
  I_RESULTS,
  I_TITLE,
  I_DESCRIPTION,
  I_CONTEXT,
 /* I_DONE,  uvolni, pokud chces komunikovat i oddelovace polozek */
  I_NONE /* ukonceni seznamu */
};

char *_error_string[] =
/* !!! poradi je vazano na enum v cross.h !!!! */
{
  "Nedoslo k zadne chybe.",
  /* fatal */
  "Neni dostatek volne operacni pameti.",   //OUT_OF_MEMORY,
  "Nepodarilo se nacist konfiguracni soubor.", //CANT_READ_CONFIG,
  /* errs while reading config from file/stdin ... fatal */
#define MISCONF  "Chyba v konfiguraci: "
  MISCONF "nezname jmeno volby.",          //MISCONF_UNKNOWN_CONFIG_NAME,
  MISCONF "modul nepodporuje danou volbu.", //_UNSUPPORTED_CONFIG_NAME,
  MISCONF "ocekavana hodnota typu integer.", //_INTEGER_EXPECTED,
  MISCONF "ocekavana 1 nebo 0.",            //_BOOLEAN_EXPECTED,
  MISCONF "neznama znakova sada",          //_UNKNOWN_CHARSET,
  MISCONF "neudan retezec ukoncujici popis stranky.", //_INCLUDE_STOP_UNSPECIFIED,
  MISCONF "neocekavany konec konfiguracniho souboru.", //_UNEXPECTED_END_OF_FILE,
  MISCONF "yaaho dosud nepodporuje udani popisu stranky odkazem.", //_EXTERNAL_INCLUDES_UNSUPPORTED,
  MISCONF "interni chyba.",                   //_INTERNAL_ERROR,
  
  /* when yaaho examines query */
  "Prehled pozadavku (QUERY_STRING) nebyl nalezen, nevim, co mam delat.",   //QUERY_NOT_FOUND,
  "Musite zadat alespon jeden vzorek k vyhledani.",  //NO_SAMPLES_SPECIFIED,
  
  /* when plotter reads items */
  "Seznam nalezu je poskozen, plotter jej nemuze prevest do HTML.", //DAMAGED_INTCOM_STREAM,
  "Seznam nalezu nebyl radne ukoncen.",      //INTCOM_STREAM_UNEXPECTED_EOF,
  /*revorable*/
  "Soubor se neporadilo otevrit.",    //OPEN_FAILED,
  "O souboru se nepodarilo zjistit dalsi informace.", //GET_INFO_FAILED,
  "Chyba cteni (I/O error),",      //IO_ERROR

  "Nepodarilo se spojeni na navazujici proces searcher.", // SEARCHER_EXEC_FAILED
  "Nepodarilo se spustit prohledavani souboru (prikaz find).",  // FIND_EXEC_FAILED
  "Nepodarilo se spojeni na navazujici proces plotter." // PLOTTER_EXEC_FAILED
};

prefatal_state_t prefatal_state; 

int fatal_flush(FILE *output, int err)
/* v pripade fatalniho selhani do intkomu vyplive chybejici ukoncovace a pak hlasku chyby */
#define PRINT(WHAT) fprintf(output, (WHAT "\n"))
{
  switch (prefatal_state)
   {
    case not_linked:
      return 1;
    case nothing_written:
      PRINT(CONFIG_FINISHED);
    case config_written:
      PRINT(SAMPLES_FINISHED);
    case samples_written:
      PRINT(ALL_FILES_FINISHED);
    case files_written:
      /* uz zadny dalsi oddelovac neni treba */
      ;
   }
  
  fprintf(output, FATAL_FORMAT "%i %s\n", err,error_string(err));
  
  exit(1);
};

void
check_input_for_fatal(FILE *input)
{
  char line[INTCOM_LINE_LEN+1];
  int err;
  
#ifdef dbgprt
fprintf(stderr,"Checking for fatal notification: ");
#endif
  while ( my_fgets(line,INTCOM_LINE_LEN,input) != NULL )
   {
#ifdef dbgprt
fprintf(stderr,">>>%s<<<\n",line);
#endif
    if (sscanf(line,FATAL_FORMAT "%i", &err))
      fatal(err);
   }
#ifdef dbgprt
fprintf(stderr,"Nothing.\n");
#endif
}





/* --- veci k znakove sade --- */
char internal_undia_table[256] =
/* tabulka pismeno->totez pismeno velke a bez diakritiky v kodu
   interni znakove sady, tj. INTERNAL_CHARSET
   Nyni je to v ISO-8859-2. */
{
  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16,
  17, 18, 19, 20, 21, 22, 23, 24, 25, 26, 27, 28, 29, 30, 31, 32,
  33, 34, 35, 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48,
  49, 50, 51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64,
  65, 66, 67, 68, 101, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80,
  81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 91, 92, 93, 94, 95, 96,
  65, 66, 67, 68, 101, 70, 71, 72, 73, 74, 75, 76, 77, 78, 79, 80,
  81, 82, 83, 84, 85, 86, 87, 88, 89, 90, 123, 124, 125, 126, 127,
  128, 129, 130, 131, 132, 133, 134, 135, 136, 137, 138, 139, 140,
  141, 142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 153,
  154, 155, 156, 157, 158, 159, 160, 65, 162, 76, 164, 76, 83, 167,
  168, 83, 83, 84, 90, 173, 90, 90, 176, 65, 178, 76, 180, 76, 83,
  183, 184, 83, 83, 84, 90, 189, 90, 90, 82, 65, 65, 65, 65, 76, 67,
  67, 67, 101, 101, 101, 101, 73, 73, 68, 68, 78, 78, 79, 79, 79, 79,
  215, 82, 85, 85, 85, 85, 89, 84, 83, 82, 65, 65, 65, 65, 76, 67,
  67, 67, 101, 101, 101, 101, 73, 73, 68, 68, 78, 78, 79, 79, 79, 79,
  247, 82, 85, 85, 85, 85, 89, 84, 255
};

int ltr_cmp(letter a, letter b)
{
  return (a==b ? 0 : 1);
};

int ltr_cmpi(letter a, letter b)
/* porovna znaky a a b (v interni znakove sade, tj INTERNAL_CHARSET) bez ohledu
   na velikost a diakritiku. K tomu potrebuje internal_undia_table vyrobenou
   rovnez pro interni znakovou sadu! */
{
  return (internal_undia_table[a]==internal_undia_table[b] ? 0 : 1);
};


/* --- obecne --- */

#include <string.h>
char *
get_config_file_name(int argc, char *argv[])
/* zpracuje optiony a vycucne z nich jmeno konfig. souboru,
   nebo vrati NULL*/
{
  int i;
  for (i=1; i<argc; i++)
  {
    if (0 == strcmp(argv[i],"-h"))
      help_and_quit();
    if (0 == strcmp(argv[i],"-c"))
     {
      i++;
      if (i<argc)
	return argv[i];
     }
  }
  return NULL;
}


