/* field.c
   prace s dynamicky alokovanymi poli
*/

/* #define TEST */

#include "field.h"
#include <stdlib.h>

/*
typedef struct _field
{
  int items_count;
  int items_allocated;
  int allocation_block;
  void *items[];
} field;
*/

field *
fld_new(int allocation_block)
{
  field *f;

  f=(field*)malloc(sizeof(field));
  if (f==NULL)
    return f;

  f->allocation_block=allocation_block;

  f->items=(void**)malloc(sizeof(void *) * allocation_block);
  if (f->items == NULL)
   {
    free(f);
    return NULL;
   }

  f->items_count=0;
  f->items_allocated=allocation_block;

  return f;
}




int
fld_add_item(field *f, void *itm)
/* vraci idx nove polozky pole nebo -1 */
{
  if (f == NULL) return -1;

  f->items_count++;

  if (f->items_count > f->items_allocated)
   {
    // nutno realokovat
    void **n_items;

    n_items=(void**)realloc(f->items,sizeof(void*) * (f->items_allocated+f->allocation_block));

    if (n_items == NULL)
     {
      f->items_count --;
      return -1;
     }
     
    /*memcpy(nMachine, mMachine, sizeof(mMachine));
    delete[] mMachine;*/
    
    f->items = n_items;
    f->items_allocated+=f->allocation_block;
   };
  
  fld_set_item(f, f->items_count -1, itm);

  return (f->items_count-1);
}

void *
fld_item(field *f, int idx)
/* vraci polozku idx nebo NULL, kdyz je idx mimo rozsah */
{
  if (f == NULL ||
      f->items_count<idx ||
      idx<0)
    return NULL;
  
  return f->items[idx];
}

void
fld_set_item(field *f, int idx, void *itm)
{
  if (f==NULL ||
      f->items_count<idx ||
      idx<0)
    return;

  f->items[idx] = itm;
}

#ifdef TEST
#include <stdio.h>
#endif

void fld_internal_quicksort(field *f, int (*comparator)(void*, void*),
			    int a, int b)

#define SWAP(A,B)                         \
  {                                       \
    swap=fld_item(f,(A));                 \
    fld_set_item(f,(A), fld_item(f,(B))); \
    fld_set_item(f,(B), swap);            \
  }

{
  int i,j, pivot;
  void *med, *swap;

#ifdef TEST
  printf("QS entry: ");
  for (i=0; i<fld_items_count(f); i++)
    printf("%d ", *((int*)fld_item(f,i)));

  printf("\n");
#endif

  if (a>=b) return;
  if (b-a == 1)
   {
    if ( comparator(fld_item(f,a),fld_item(f,b)) >0 )
     SWAP(a,b);
    return;
   }

  i=a+1; j=b;
  med=fld_item(f,a);

  while (i<j)
   {
    while (i<j && comparator(fld_item(f,i),med) <= 0)
      i++;
    while (i<j && comparator(fld_item(f,j),med) >= 0)
      j--;

    if (i<j)
     SWAP(i,j);
   } /* i==j */
  if ( comparator(fld_item(f,i),med) < 0 )
   {
    SWAP(a,i);
    pivot=i;
   }
  else
   {
    SWAP(a,i-1);
    pivot=i-1;
   }

  fld_internal_quicksort(f,comparator,a, pivot-1);
  fld_internal_quicksort(f,comparator,pivot+1, b);

  return;
}


#ifdef TEST

#include <stdio.h>

void printallfld(field *f)
{
  int i;

  printf("All: ");
  for (i=0; i<fld_items_count(f); i++)
    printf("%d ", *((int*)fld_item(f,i)));

  printf("\n");
}

int intcomp(void *a, void *b)
{
  return (*(int*)a-*(int*)b);
}

void main()
{
  field *f;

  f=fld_new(3);

  int i, *box;

  /*randomize();*/

  for (i=0; i<10; i++)
   {
    box=(int*)malloc(sizeof(int));
    *box=random(100);
    fld_add_item(f,box);
    printallfld(f);
   }

  fld_quicksort(f,&intcomp);

  printallfld(f);
}


#endif /*TEST*/

