/* items.c */


#include "items.h"
#include "cross.h"
#include <stdio.h>
#include <time.h>
#include "field.h"
#include "charset.h"
#include "samples.h"

#define ITEM_ALLOCATION_BLOCK  30

#define SUBSTITUTION_LEN 30
/* vyrazy __NECO__ v layoutech omezeny timto */

static field *items;
/* hlavni sklad nactenych vzorku */


void init_items()
{
  items = fld_new(ITEM_ALLOCATION_BLOCK);
  if (items == NULL)
    fatal(OUT_OF_MEMORY);
}

int item_comparator(void *i1, void *i2)
{
  return -( ((item*)i1)->samples_matched - ((item*)i2)->samples_matched );
}

void sort_items()
{
  fld_quicksort(items, &item_comparator);
};



/* ---- reading */

#include <stdlib.h> /*malloc*/
#include <string.h> /*strtok*/
#include "strtools.h" /*WHITE_SET*/

#include "config.h"

char *dup_path_to_url(char *path)
/* prevede cestu souboru na adresu zvenci */
/* FIXME pokud dirdelimiter neodpovida rootdir, tak se to kouse!!! */
{
  char *file_name;
  char *root_dir=config_str(C_ROOT_DIR);
  char dir_delim=*config_str(C_DIR_DELIMITER);
  char *url;

  char *p;

  file_name = strstepstr(path, root_dir);
  if (file_name==NULL)
    /* bude mu stacit pripojit */
    file_name = path;

  /* udelej dobre oddelovace URL */
  strchrrepl(file_name,dir_delim,'/');

  if ( root_dir[strlen(root_dir)-1] == *file_name
       && *file_name == dir_delim )
    /* potlacit nasobna lomitka */
    file_name++;

  url=(char*)malloc( strlen(file_name)+
		     strlen(config_str(C_ROOT_URL))+1);

  *url=0;
  strcat(url,config_str(C_ROOT_URL));
  strcat(url,file_name);

  return url;
}



item *new_item()
{
  item *itm;

  itm=(item*)malloc(sizeof(item));
  if (!itm) return NULL;

  itm->url=NULL;
  itm->modified=0;
  itm->charset=CH_UNKNOWN;
  itm->title=NULL;
  itm->description=NULL;
  itm->context=NULL;
  itm->samples_matched=0;
  return itm;
}

int
read_item(FILE *input, char *line)

#define DUPCONVSTR(DEST,SRC)                         \
  {                                              \
    (DEST) = (char*)malloc(sizeof(char) * (strlen(SRC) +1));               \
    if ( (DEST) == NULL ) return OUT_OF_MEMORY;  \
    if (INTERNAL_CHARSET != config_int(C_CHARSET_NETWORK_OUTPUT))  \
      charset_convert_string( (SRC), (DEST), (DEST)+strlen(SRC)+1,  \
             INTERNAL_CHARSET, config_int(C_CHARSET_NETWORK_OUTPUT));  \
    else                                 \
      strcpy((DEST),(SRC));   \
  }

{
  item *itm;
  int i, cmd;
  int err;
  
  int c=0;
  char *p;


  itm=new_item();
  if (itm == NULL)
    return OUT_OF_MEMORY;

  for (i=0;
       I_NONE != (cmd = intcom_item_order[i]);
       i++)
   {
#ifdef dbgprt
fprintf(stderr,"plotter: readitem phase %i.\n",i);
#endif
    switch (cmd)
     {
      case I_PATHNAME:
	itm->url=dup_path_to_url(line);
	break;

      case I_STAT:
	if (0 == sscanf(line,"%ld",&(itm->modified)) )
	  return DAMAGED_INTCOM_STREAM;
	break;

      case I_CHARSET:
	if (0 == sscanf(line,"%d",&(itm->charset)) )
	  return DAMAGED_INTCOM_STREAM;
	break;

      case I_RESULTS:
	for ( p=strtok(line, WHITE_SET);
	      p ;
	      p=strtok(NULL, WHITE_SET))
	 {
	  if ( 0 == sscanf(p,"%d",&c) )
	    return DAMAGED_INTCOM_STREAM;
	  itm->samples_matched += c;
	 }
	break;

      case I_TITLE:
	DUPCONVSTR(itm->title,line);
	break;

      case I_DESCRIPTION:
	DUPCONVSTR(itm->description,line);
	break;

      case I_CONTEXT:
	DUPCONVSTR(itm->context,line);
	break;

      case I_DONE:
	if (0 != strcmp(line,FILE_FINISHED))
	  return DAMAGED_INTCOM_STREAM;
	break;

     } /* switch cmd */

    if (intcom_item_order[i+1] != I_NONE)
      /* jeste priste budeme neco chtit, nacist dalsi buffer */
     {
      line = my_fgets(line, INTCOM_LINE_LEN, input);
#ifdef dbgprt
fprintf(stderr,"plotter: reading [%s].\n",line);
#endif
     }
   }

  err=fld_add_item(items,itm);
  return ( err==-1 ? OUT_OF_MEMORY : 0 );

}


int
read_items(FILE *input)
/* nacte polozky do pameti */
{
  char buff[INTCOM_LINE_LEN];
  char *line;
  int err;

#ifdef dbgprt
fprintf(stderr,"plotter: reading items.\n");
#endif
  while (NULL != (line = my_fgets(buff, INTCOM_LINE_LEN, input)) )
   {
#ifdef dbgprt
fprintf(stderr,"plotter: reading [%s].\n",line);
#endif
    if (0 == strcmp(line, ALL_FILES_FINISHED) )
      return 0;
#ifdef dbgprt
fprintf(stderr,"plotter: calling readitem.\n");
#endif
    err = read_item(input, line);
#ifdef dbgprt
fprintf(stderr,"plotter: readitem returned %i.\n",err);
#endif
    if (err)
      return err;
   }

  return INTCOM_STREAM_UNEXPECTED_EOF;
}





/*  flushing ---- */


item *global_item_for_substitutor = NULL;
/* pouziva substitutor pri flushovani polozek */

#include "charset.h"

void
flush_http_header(FILE *output)
{
  char *chname;
  
  chname=charset_to_a(config_int(C_CHARSET_NETWORK_OUTPUT), charset_name_MIME);
  if (config_int(C_CHARSET_ADD_TO_CONTENT_TYPE) && chname)
    fprintf( output, "Content-type: text/html; charset=%s\n\n",chname);
  else
    fprintf(output,"Content-type: text/html\n\n");
}




int header_substitutor(FILE *output,char *str)
/* nahrazuje retezce __NECO__ v hlavicce vystupu */
{
  if (0==strcmp("YAAHO_CGI",str))
    fputs(config_str(C_YAAHO_CGI_URL), output);
  else
  if (0==strcmp("ORIGINAL_REQUEST", str))
    flush_samples_string(output);
  else
    /*FIXME - mohl by jeste reagovat na PAGES */
    return 0;

  return 1; /* uspesne nahrazeni */
}

int item_substitutor(FILE *output,char *str)
/* nahrayuje retezce __NECO__ v jedne polozce */
#define FLUSH(STR) { if (STR) fputs((STR),output); }
{
  if (0 == strcmp("URL",str))
    FLUSH(global_item_for_substitutor->url)
  else
  if (0 == strcmp("TITLE",str))
   {
    if (global_item_for_substitutor->title && 
        *skipwhite(global_item_for_substitutor->title))
      fputs(global_item_for_substitutor->title,output);
    else
      FLUSH(global_item_for_substitutor->url);
   }
  else
  if (0 == strcmp("DESCRIPTION",str))
    FLUSH(global_item_for_substitutor->description)
  else
  if (0 == strcmp("MATCHES",str))
    printf("%d",global_item_for_substitutor->samples_matched);
  else
  if (0 == strcmp("CONTEXT",str))
    FLUSH(global_item_for_substitutor->context)
  else
  if (0 == strcmp("DATE",str))
    FLUSH(ctime(&(global_item_for_substitutor->modified)))
  else
  if (0 == strcmp("CHARSET_MIME",str))
    FLUSH(charset_to_a(global_item_for_substitutor->charset,charset_name_MIME))
  else
  if (0 == strcmp("CHARSET_SHORT",str))
    FLUSH(charset_to_a(global_item_for_substitutor->charset,charset_name_short))
  else
    return 0; /* vic neumime nahradit */

  return 1;
}



void flush_buffer(FILE *output,char *buffer,
		  int (*substitutor)(FILE *output,char*))
/* vysype buffer\0, na retezce __NECO__ zavola substitutor NECO */
/* buffer musi jiz byt ve vystupni znakove sade */
{
  char *a, *b, *c;

  a=buffer;

  while (*a)
   {
    b=strstr(a, "__");
    if (b==NULL)
      break;

new_c:
    c=strstr(b+2, "__");
    if (c==NULL)
      break;
      /* FIXME, pokud by se mohlo stat, ze nahrada je v na hranici bufferu */
      /*  to se ale nestane, protoze cely soubor k plivani je povinne nacten v bufferu */

    if (b-c > SUBSTITUTION_LEN)
     {
      b=c;
      goto new_c;
     }

    *b=0; fputs(a,output); *b='_';
    /* vyplivni kus pred nahrazovanim, buffer ale neposkod */

    *c=0;
    if (substitutor(output,b+2))
      a=c+2;
    else
     {
      fputs(b,output);
      a=c;
     }
    *c='_';
    /* vyplivni __NECO_CO_SUBSTOR_NEVZAL a zkus se chytit od __ dal */
   }

  fputs(a,output); /* doplivni konec bufferu */

  return;
}


void
flush_items(FILE *output)
{
  int i, cnt;
  item *itm;

  flush_http_header(output);

  flush_buffer(output,config_str(C_LAYOUT_ANSWER_PAGE_HEADER), header_substitutor);

  if (fld_items_count(items) <= config_int(C_REQUEST_COUNT_LIMIT) )
    cnt=fld_items_count(items);
  else
    cnt=config_int(C_REQUEST_COUNT_LIMIT);

  for (i=0; i<cnt; i++)
   {
     global_item_for_substitutor = (item*)fld_item(items, i);
     flush_buffer(output,config_str(C_LAYOUT_ANSWER_PAGE_ITEM), item_substitutor);
   }

  flush_buffer(output,config_str(C_LAYOUT_ANSWER_PAGE_FOOTER), header_substitutor);

  return;
}


