/*
   machine.c
*/

#include "machine.h"
#include <stdlib.h> /*malloc*/


#define MACHINEBLOCKSIZE        30
//velikost alokacniho bloku na stavy masinky - mereno v poctu stavu


machine *dia_mach;
machine *asc_mach;



void init_machine(machine *m)
{
  m->state_count=0;
  m->states_allocated=-1;
  m->states = NULL;
  m->current_node=NULL;

  return;
}

void reset_machine(machine *m)
/* dostavenou masinku da do poc. stavu */
{
  m->current_node=NULL;
  return;
}


int new_state(machine *m)
// prida novy stav k automatu, vrati jeho ID
{
  if (m->state_count>m->states_allocated)
  {
    // Musime alokovat novy blok
    if (m->states_allocated == -1)
    {
      // Dosud nebylo vubec alokovano
      mach_state *states; // pole stavu

      states=(mach_state*)malloc(MACHINEBLOCKSIZE*sizeof(mach_state));

      if (states==NULL) fatal(OUT_OF_MEMORY);

      m->states=states;
      m->states_allocated=MACHINEBLOCKSIZE;
    }
    else
    {
      // nutno realokovat
      mach_state *re_states;
      re_states=(mach_state*)realloc(m->states,(m->states_allocated+MACHINEBLOCKSIZE)*sizeof(mach_state));

      if (re_states==NULL) fatal(OUT_OF_MEMORY);
      /*memcpy(nMachine, mMachine, sizeof(mMachine));
      delete[] mMachine;*/
      m->states=re_states;
      m->states_allocated+=MACHINEBLOCKSIZE;
    };
  };

  m->state_count++;
  // zvysime pocet stavu v automatu
  // vime, ze muzem zapsat odkaz na stav do Machine[state_count-1];
  FIRSTNODE(m,m->state_count-1)=NULL;

  return (m->state_count-1);
  // dej state_count k dispozici, zvys hranici
};

mach_node *newnode(letter _ltr, int _nextstate, mach_node *_next)
// vzrobi krabicku s uzlem do masinky
{
  mach_node *node;

  node=(mach_node*)malloc(sizeof(mach_node));
  if (node == NULL) fatal(OUT_OF_MEMORY);

  node->ltr=_ltr;
  node->data.nextstate=_nextstate;
  node->next=_next;

  return node;
};

void add_node_to_state(machine *m, mach_node *node, int state)
// pripoji JAKO POSLEDNI k danemu stavu
{
  mach_node *n;

  n=FIRSTNODE(m,state);
  if (n == NULL)
   {
    FIRSTNODE(m,state)=node;
    return;
   };

  for (; n->next == NULL; n=n->next);
  n->next=node;
};

void add_node_at_state(machine *m,mach_node *node, int state)
// pripoji JAKO PRVNI k danemu stavu novy uzel
{
  node->next=FIRSTNODE(m,state);
  FIRSTNODE(m,state)=node;
  return;
};


void
add_sample(machine *m, int inSampleID, const letter* inSample,
           int (*comparator)(letter,letter))
{

  int   state;
  mach_node      *node;

  // zalozit masinku, neni-li
  if (m->states_allocated == -1) state=new_state(m);

  state=0; // zacni hledat od zacatku

  // nejdriv ujdeme, co uz zname
  node=FIRSTNODE(m,state);
  while (node  && *inSample)
  {
    if ( 0 == comparator(node->ltr,*inSample) )
    {  // je tam zname pismeno
      inSample++;
      state= node->data.nextstate;
      node= FIRSTNODE(m,state);
    }
    else
    {  // zkusme, jinou cestu pro toto pisnemo
      node= node->next;
    };
  };
  // bud jsme vycerpali vzorek (a state je stav odpovidajici celemu vzorku
  // nebo jsme vylezli mimo zname cesty (node==NULL a state je ten, kam az se to shodovalo

  for(;*inSample; inSample++)
  { // jeste je, co naucit, nauc to
    // vyrob novou hranu
    int ns;  // cislo noveho stavu, kam se ma prejit pismenem *inSample
    mach_node *n, *p;

    ns=new_state(m);
    if (ns==-1) fatal(OUT_OF_MEMORY);

    node=newnode(*inSample, ns, NULL);

    // zapoj ji
    if ((n=FIRSTNODE(m,state))==NULL )
    {
      m->states[state]=node; // je jediny, proste ho tam pripoj
    }
    else {
      for (n=FIRSTNODE(m,state); n->next && n->ltr==FLUSHLETTER; n=n->next) {;};
      // dojed az za vsechny pokyny k flushovani
      // n je ted ten, za ktery chceme node pripojit
      node->next=n->next;
      n->next=node;
    };

    state=node->data.nextstate;
  };

  // state je ted stav odpovidajici celemu nastenemu vzorku, pridejme pokyn
  // k flushovani
  add_node_at_state(m,newnode(FLUSHLETTER,inSampleID, NULL), state);

};


#include "que.h"

int
walk(machine *m, int state, letter ltr, int (*comparator)(letter,letter))
// zjisti, kam se ze stavu state prechazi pismenem ltr, jede i po zachrannych
// funkcich
{
  mach_node *node;

  for(node=FIRSTNODE(m,state); node; )
   {
    
    if (node->ltr == CATCHLETTER)
     {
      // musime se chytit zachrany
      state= node->data.nextstate;
      node = FIRSTNODE(m,node->data.nextstate);
     }
    else if (0 == comparator(node->ltr, ltr))
        // nasli jsme spravnou cestu!
      return node->data.nextstate;
    else
      node = node->next;
   };

  return 0; // neuspels s nicim, musis jet od yacatku
}


int
scan_machine(machine *m, letter ltr, int (*comparator)(letter,letter))
/* postupuj po strojecku */
{
  mach_node *node;

  if (m->current_node == NULL)
    m->current_node = FIRSTNODE(m,0);

  for(node=m->current_node; node; )
   {
    switch (node->ltr)
     {
      case CATCHLETTER:
	 // musime se chytit zachrany
	 node = FIRSTNODE(m,node->data.nextstate);
	 break;
      case FLUSHLETTER:
	 // musime mu vratit vystup!
	 m->current_node=node->next;
	 return node->data.sampleid;
      default:
	 if (comparator(node->ltr,ltr)==0)
	 // nasli jsme spravnou cestu!
	  {
	   m->current_node=FIRSTNODE(m,node->data.nextstate);
	   return NOTHING_NEW;
	  }
	 else
	  node = node->next;
     } /*switch*/
   }

  m->current_node=FIRSTNODE(m,0);

  return NOTHING_NEW; /* zadnou cestu jsme nenasli, drzime se v nule */

}


void copy_output_nodes(machine *m,int srcstate, int deststate)
{
  mach_node *node;

  for (node=FIRSTNODE(m,srcstate);
       node && (node->ltr == FLUSHLETTER);
       node=node->next )
   {
    add_node_at_state(m,newnode(FLUSHLETTER,node->data.sampleid,NULL),deststate);
   };
};


#ifdef DEBUG

#include "iostream.h"
void print_machine(machine *m)
{
	mach_node	*node;
	int			i;

	cout<< "---" << nl<<nl;

	for (i=0; i<m->state_count; i++)
	{	// vytiskni stavajici stavy
		node=FIRSTNODE(m,i);
		cout << "[" << i << "]  ";
		while (node)
		{
			if (node->ltr == FLUSHLETTER)
			  cout << "  -"<< node->data.sampleid <<"-$";
			else
			  cout << "  -(" << node->ltr << ")-> [" << node->data.nextstate<<"]";


			node= node->next;
		};
		cout <<nl;
	};
};
#endif


void
finish_machine(machine *m, int (*comparator)(letter,letter))
// dodela ve strukture mMachine zachranne odkazy pro nedefinovana pismena
// pracuje vlnou pres stavy
// nakonec maji vsechny stavy na konci sveho retizku prechodu pripojeny
// zachranny uzel (CATCHLETTER,newstate)
// v kazde fazi vlny je tento uzel na zacatku a postupne se presune na konec retizku
{
  que      q;    // fronta uzlu k zpracovani
  mach_node *node; // pracovni uzel
  int      savestate; // zachranny stav zpracovavaneho stavu
  mach_node *son;  // syn zpracovavaneho stavu

  int t;
  mach_node *lastnode;

  emptyque(&q);

  // zachranny uzlicek pro nulty stav vubec nedefinuji

  for(node=FIRSTNODE(m,0); node; node=node->next)
   // pres vsechy syny nulteho stavu
    if (node->ltr != FLUSHLETTER)
     // mimo falesnych uzlu
     {
      add_node_at_state(m,newnode(CATCHLETTER,0,NULL), node->data.nextstate);
      // pripoj zachranny stav - skok do nulteho stavu

      quepush(&q,(void*)FIRSTNODE(m,node->data.nextstate));
      // a dej do fronty odkaz na toho syna
     };


  for (node=(mach_node*)quepop(&q); node; node=(mach_node*)quepop(&q))
   {
    // node je prvni uzel stavu, ktery prave probirame
    // z IP je to rovnou zachranny uzel
#ifdef DEBUG
    if (node->ltr != CATCHLETTER) {cout<<"IPporusen";exit(1);};
#endif

    savestate=node->data.nextstate;
    // zachranny stav jsme si zapamatovali

    son=node->next;
    // son je popis hrany, kterou ted chceme zvladnout
    if (son == NULL) continue; // getting from queue
    // pokud zadna hrana neexistuje, mame vystarano

    //ted je potreba uzlik se zachrannym stavem odpojit a nakonec ho
    //pripojit na konec
    node->ltr      = son->ltr;
    node->data.nextstate = son->data.nextstate;
    node->next     = son->next;

    // son je ted uzlik k premazani, nechame si ho ale pro pripojeni na konec
    son->ltr      = CATCHLETTER;
    son->data.nextstate = savestate;
    son->next     = NULL;
    // son je hotova zachranka, idealni k pripojeni na konec

    lastnode=node;
    // node je naopak prvni uzel, ktery musime projit
    for ( ; node ; node=node->next)
     {
      if (node->ltr == FLUSHLETTER)
       // tohle neni prava hrana
       { lastnode=node;
	 continue;
       };

      // node->state je stav, ktery potrebuje pridat zachrannou fci

      t=walk(m, savestate,node->ltr, comparator);
      // t je zachranny stav pro zpracovavanou hranu node->newstate

      /*copy_output_nodes(m, t, node->data.nextstate);
      // rozsir vystupni funkci stavu node->data.nextstate o vystupy stavu t
       Odstraneno, s ohledem na implementaci strojecku, pri prochazeni
      ve funkci scan_machine se tak jako tak nejdrive narazi na vsechny
      hlasky vystupu a pak teprve se najde, jak se ma zachranovat.
      Kdyby se navic u zachranneho stavu pridaly informace o hlaskach
      zachranovaneho stavu, hlasilo by se dvakrat.
      */


      add_node_at_state(m, newnode(CATCHLETTER,t,NULL),node->data.nextstate);
      // stavu node->newstate jsme pridali zachrannou fci

      quepush(&q,FIRSTNODE(m,node->data.nextstate));
      // stav pridejme do fronty

      lastnode=node; // po protreby pripojovani zachranky na konec seznamu
     }; // zpracovani vsichni synove tohoto stavu

    // lastnode je posledni uzel zpracovavaneho stavu, je treba pripojit
    // zan zachranku pripravenou v uzlu son


    lastnode->next = son;

   }; // cela fronta je prazdna

  // zpracovali jsme celou frontu, zachranne fce hotovy

  return;
};

