/*
   Machine.h

   prace se strojeckem AHO-CORASICK
*/

/*#define DEBUG*/

#include "cross.h" /* typedef letter */

typedef struct _MachNode
{
	letter	ltr;			// psmeno, pi nm se pechz
					// nebo spec. hodnoty:
					//  CATCHLETTER  e je to zchrann odkaz
					//  FLUSHLETTER  e je to vstupn vzorek
	union
	{
		int		nextstate;		// index novho nebo zchrannho stavu
		int		sampleid;	// index vzorku, kter vystupuje
	} data;
	struct _MachNode	*next;	// odkaz na nov stav
} mach_node;

typedef mach_node *mach_state;

typedef struct _machine
{
  mach_node *current_node;
  int state_count;
    // aktualni pocet stavu, tolik stavu masinka ma
  int states_allocated;
    // limit poctu stavu, aniz by bylo nutne zvetsovat pole odkazu
    // -1 znamena, za zadne pole odkazu jeste alokovano nebylo
  mach_state *states; // pole stavu
} machine;
// definice celho strojeku

#define FIRSTNODE(MACHINE,ID) (MACHINE)->states[(ID)]


#define FLUSHLETTER             -1
// falesne pismeno, znaci, ze se ma vytisknout vzorek
#define CATCHLETTER             0
// zachytny stav, nenajde-li se cesta pres dane pismenko, chytneme se teto


void init_machine(machine *m);

void add_sample(machine *m, int inSampleID, const letter* inSample,
                int (*comparator)(letter,letter));
/* prida vzorek k automatu */

void finish_machine(machine *m, int (*comparator)(letter,letter));

void reset_machine(machine *m);
/* dostavenou masinku da do poc. stavu */

#define NOTHING_NEW -1

int scan_machine(machine *m, letter ltr, int (*comparator)(letter,letter));
/* z aktualniho stavu prejde pres pismeno, zastavi se bud na spravnem stavu
   nebo na prvnim vystupnim uzlu
   vraci cislo id vzorku nebo NOTHING_NEW
*/

/* pravidla pro uzivani komparatoru na pismenka:
    1. overit, je-li node->letter opravdu pismeno (tj. neni CATCHLETTER ani
       FLUSHLETTER)
       overovat beznym ==
    2. pak teprve se ptat na rovnost s nalezenym pismenem pomoci komparatoru
*/

#ifdef DEBUG
void print_machine(machine *m);
#endif




extern machine *dia_mach;
extern machine *asc_mach;
