/* myexec.c udela exec, ale argumenty a program zadany jako commandlina

  umi to zpracovat argument v uvozovkach, ale nedela zadne dalsi expanze
*/

#include <unistd.h>
#include <fcntl.h>
#include "myexec.h"
#include "strtools.h"

//  #define DEBUG

#ifdef DEBUG
#include <stdio.h>
#include <errno.h>
#endif DEBUG



int pipe_to_fork_pip[2];


char **
build_argv(char *cmd)
/* rozebere radku a postavi z ni nove pole argv, v pripade neuspechu vraci NULL */
{
  char *p,*q, *k, *l;
  char sentinel;
  char **argv;
  int cnt, i, len;
  
  /* spocti, kolik je argumentu */
  p=skipwhite(cmd);
  cnt=-1;
  while (*p)
   {
    cnt++;
    if (*p == '"' || *p == '\'')
     {
      /* argument v zabalen */
      sentinel = *p;
      p++;
      q=strchr(p, sentinel);
      while (q && *q && *(q-1)=='\\')
        q=strchr(q+1, sentinel);
      if (!q)
        break;
     }
    else
     {
      q = p + strcspn(p, WHITE_SET);
     }
    p = *q ? q+1 : q;
   }
  if (cnt < -1)
    return NULL;
    /* pitomec nam nedal zadny string! */
  
  /* nastav jednotlive argumenty */
  argv = (char **)malloc((cnt+1) * sizeof(char*));
  if (!argv)
    return NULL;
  
  p=skipwhite(cmd);
  i=0;
  while (*p)
   {
    sentinel = 0; /* kvuli cistote */
    if (*p == '"' || *p == '\'')
     {
      /* argument v zabalen */
      sentinel = *p;
      p++;
      q=strchr(p, sentinel);
      while (q && *q && *(q-1)=='\\')
        q=strchr(q+1, sentinel);
      if (!q)
        break;
     }
    else
     {
      q = p + strcspn(p, WHITE_SET);
     }
    /* ted je ten argument mezi p a q a sentinel je pripadne uvozovka nebo apostrof */
    len = q-p;
    
    argv[i] = (char*) malloc(sizeof(char)*(len+1));
    if (!argv[i])
      return NULL;
      /* FIXME s dealokaci se moc nepare */
    /* prijde kopirovani, s tim, ze escapnute \" se nahrazuji */
    for (k=p, l=argv[i];
         k-p < len;
         k++, l++)
     {
      if (*k == '\\' && *(k+1) == sentinel) /* sentinel muze byt nula, ale *l nemuze, takze ok */
        k++;
      *l=*k;
     }
    *l = 0; /* ukoncovaci nula */
    
#ifdef DEBUG
fprintf(stderr,"%d: %s\n",i, argv[i]);
#endif DEBUG
    
    i++; /* pocet argumentu */
    
    if (sentinel && *q == sentinel)
      q++;
    p = *q ? q+1 : q;
    p = skipwhite(p);
   }
  
  argv[i]=NULL;
  
  return argv;
}


int
able_to_exec(char *cmd)
/* testuje, jestli muzu spustit cmd
   vraci 0 pokud ano, -1 pokud ne a errno rika proc */
{
  int len;
  char *p, pgm[PATHNAME_LEN];
  
  p=skipwhite(cmd);
  len=strcspn(p,WHITE_SET);
  if (len==0)
    len= strlen(p);

  strncpy(pgm,p,len); *(pgm+len)=0;  
  
#ifdef DEBUG
fprintf(stderr, "Cheching access to >>>%s<<<.\n",pgm);
#endif
  
  return access(pgm,X_OK);
}



int
my_execp(char *cmd)
/* spousti jako-command-linu */
{
  char **argv;
  
  argv=build_argv(cmd);
  if (! argv) return;
  
  return execvp(argv[0],argv);
}




int
pipe_to_fork_father(pid_t son)
{
#ifdef DEBUG
  fprintf(stderr,"Father: Son was launched, son's PID is %i.\n",son);

  fprintf(stderr,"Father: Piping stdout to son's stdin.\n");
#endif

  close(pipe_to_fork_pip[0]);
  /* zavri ten cteci konec roury */
  close(1);
  dup(pipe_to_fork_pip[1]);
  close(pipe_to_fork_pip[1]);

  /* stdout redirected to son! */

#ifdef DEBUG
  fprintf(stderr,"Father:Succesfully lauched son and redirected stdout. Ready to go.\n");
#endif
  
  return 0;
}

int
pipe_to_fork_son(char *cmd)
{
  int err;

#ifdef DEBUG
  fprintf(stderr,"Son: I was lauched.\n");
  
  fprintf(stderr,"Son: Piping my stdin to father's stdout.\n");
#endif

  close(pipe_to_fork_pip[1]);
  /* zavri ten psaci konec roury */
  close(0);
  dup(pipe_to_fork_pip[0]);
  close(pipe_to_fork_pip[0]);

#ifdef DEBUG
  fprintf(stderr,"Son: executing >>>%s<<<.\n",cmd);
#endif
  
  err=my_execp(cmd);
  /* never returns, if successful */
  
#ifdef DEBUG
  fprintf(stderr,"Son: exec failed with %i.\n",errno);
#endif DEBUG
  
  return -2;
}



int
pipe_to_fork(char *cmd)
/* forkne se, propoji rouru otec.stdout->syn.stdin
   a v synovskem procesu spusti cmd pomoci my_execp
   
   vraci pid syna, pokud uspeje
   pokud selze vraci -1, jestlize selhalo jeste pred forkem,
                   a -2, jestlize selhalo az v synovi po forku
*/
{
  pid_t son;
  int err;
  
#ifdef DEBUG
  fprintf(stderr,"pipe_to_fork called.\n");
#endif
  
  err=able_to_exec(cmd);
  if (err == -1)
   {
#ifdef DEBUG
fprintf(stderr,"Unable to access, due %i\n",errno);
#endif
    return -1;
   }
  
#ifdef DEBUG
  fprintf(stderr,"Building pipe.\n");
#endif
  
  err=pipe(pipe_to_fork_pip);
  if (err) return -1;

#ifdef DEBUG
  fprintf(stderr,"Forking.\n");
#endif

  son=fork();
  
  if (son==1)
   {
    err=-1;
#ifdef DEBUG
    fprintf(stderr,"Forking failed, %d.\n",errno);
#endif
   }
  else if (son!=0)
   {
    err=son; /* return son's pid */
    pipe_to_fork_father(son);
   }
  else
    err=pipe_to_fork_son(cmd);
  
  return err;
}

