/*
  searcher
  vyhledavaci strojecek v baliku yaaho
*/

/* parametry prekladu: (pomoci #define)
     dos     ... bezi-li pod dosem
     dec     ... bezi na Decu
     linux   ... bezi pod linuxem
     PIPED   ... ma-li spoustet plotter samo, na zaklade konfigu
                 (nastaveno jiz v cross.h)
     dbgprt  ... globalni ladici tisky, hlavne pro rourovani
     searprt ... ladici tisky pro prohledavani
     MIdprt  ... ladici tisky pro funkce ziskavajici More Info (trebas popis dokumentu)
*/

//// #define MIdprt


#include "cross.h"
#include "common.h"
#include "machine.h"
#include "samples.h"
#include "charset.h"
#include "config.h"
#include "strtools.h" /* my_fgets */
#include <stdio.h>
#include <string.h>
#include <stdlib.h> /* exit */
#include <sys/stat.h>

#ifdef myfgetsdebug
char *myfgets_id="searcher";
#endif

#ifdef PIPED
#include "myexec.h"
#endif

#ifdef dos
#include <mem.h>
#endif

#define PATHNAME_LEN    512
// maximalni delka cesty k souboru
#define DEFAULT_CONFIG_FILE     "searcher.conf"
#define BUFFER_SIZE 10240
/* velikost bufferu pro prohledavani souboru */
typedef struct _file_info
/* uskladneni informaci o souboru, nez je vyplivnem pryc */
{
  char* title;
  char* context;
  char*description;
  int charset;
  struct stat file_stat;
} file_info;

#define CONTEXT_LEN_PREFERRED       300
/* preferovana delka kontextu pri uspesnem hledani */
#define CONTEXT_LEN      ( CONTEXT_LEN_PREFERRED <= INTCOM_LINE_LEN ?  \
                           CONTEXT_LEN_PREFERRED  : INTCOM_LINE_LEN )
/* skutecna delka kontextu nesmi presahnout delku radky v interni komunikaci, 
protoze prave jako jedina radka je kontext ze searcheru predan do plotteru */
#define DESCRIPTION_LEN  INTCOM_LINE_LEN
#define TITLE_LEN        INTCOM_LINE_LEN-2

/* --- zahlad: help a fatal */
void help_and_quit()
{
 printf(
  "Searcher: part of yaaho package" "\n"
  "obo@cuni.cz" "\n"
  "\n"
  "-h                this screen" "\n"
  "-c <config_file>  ignore default config (" DEFAULT_CONFIG_FILE ")" "\n"
  "\n"
  );
 exit(0);
};

void fatal(int kind)
{
  if (fatal_flush(stdout, kind))
    ;
  
  /* pokud jsme se dostali sem, mame za ukol hlasit chybu sami */
  
  printf("Searcher: %s\n%i\n",error_string(kind),kind);
  exit(1);
};


/* --- nacteni vzorku do pameti */

void read_samples(FILE *input)
/* ze STDIN nacte vsechny vzorky do internich struktur
   na vstupu se ocekava kazda radka samostatny vzorek
   zarazi se na SAMPLES_FINISHED
*/
{
  char line[MAX_SAMPLE_LEN]; //nactena radka
  char *line_end=line+MAX_SAMPLE_LEN;
  char *sample;
  int id; // ID vzorku, ktery seprave zapamatoval
  int required; /* je vzorek nutny? */
  char* sample_copy;

  /* na stdin se ocekava prisun vzorku v internim kodovani */
  while (my_fgets(line,MAX_SAMPLE_LEN,input))
   {
    if (0 == strcmp(line,SAMPLES_FINISHED))
      return;
      /* konec seznamu vzorku */
    if (*line=='+')
     {
      required=1;
      sample=line+1;
     }
    else
     {
      required=0;
      sample=line;
     }

    id=remember_sample(sample,required);
    /* zapamatuj si zneni vzorku */

    /* uprav masinky, aby vzorek take umely najit */
    if (config_int(C_CHARSET_NETWORK_INPUT) != CH_ASC)
     {
      /* potrebujeme obe masinky, dia i asc, neb vzorky jsou ve vice
      sadach */
      /* nejprve vzorky (uz jsou od yaaho v interni sade) dame do diamach
      */
      add_sample(dia_mach, id, sample,ltr_cmpi);
      /* a pak je odcestime a dame taky do ascmach */
      sample_copy=get_sample(id);
      charset_convert_string(sample_copy,line, line_end,
                    INTERNAL_CHARSET, CH_ASC);
      add_sample(asc_mach, id, line,ltr_cmpi);
     }
    else
     {
      /* vzorek dame jen do ascmasinky, vzorek sam je v asc... */
      add_sample(asc_mach, id, sample,ltr_cmpi);
     }
#ifdef DEBUG
print_machine(asc_mach);
if (dia_mach)
  print_machine(dia_mach);
#endif
   };
};


/* --- vystup vysledku */

void
flush_headers(FILE *output)
/* vysype vsechny potrebne hlavicky, pak uz na vystupu bude
   prehled uspesne prohledanych souboru
*/
{
  flush_config(output);
  
  prefatal_state = config_written;
  flush_samples_names(output);
  fprintf(output,"%s\n",SAMPLES_FINISHED);
  prefatal_state = samples_written;
}

void
flush_results(FILE *output,file_info *finf,char* path_name)
/* vysype na vystup informace o souboru */
#define FLUSH_STR(STR)        \
      {                       \
       if (STR)               \
        printf("%s\n",(STR)); \
       else                   \
        printf("\n");         \
      }
      /* vyplivne na radce text nebo prazdnou radku */
{
  int i, cmd;

#ifdef dbgprt
fprintf(stderr,"Searcher: mam plivnout vysledek o >>%s<<.\n",path_name);
#endif

  if (! required_samples_present())
    return;
  /* soubor bez jedineho povinneho vzorku se vubec neplive */
  for (i=0;
       (cmd=intcom_item_order[i]) != I_NONE ;
       i++)
    switch (cmd)
     {
      case I_PATHNAME:
        FLUSH_STR(path_name);
        break;
      case I_STAT:
        printf("%ld\n",finf->file_stat.st_mtime);
        break;
      case I_CHARSET:
        printf("%d\n",finf->charset);
        break;
      case I_RESULTS:
        flush_samples_counts(output);
        break;
      case I_TITLE:
        FLUSH_STR(finf->title);
        break;
      case I_DESCRIPTION:
        FLUSH_STR(finf->description);
        break;
      case I_CONTEXT:
        FLUSH_STR(finf->context);
        break;
      case I_DONE:
        FLUSH_STR(FILE_FINISHED);
        break;
     }

#ifdef dbgprt
fprintf(stderr,"Searcher: Vyplivnul jsem vysledek o >>%s<<.\n",path_name);
#endif

  return;
}

/* --- prohledavani a  studium bufferu */


void
get_context_suffix(char *dest, char *src, char *limit, int maxlen)
/* kopiruje text mimo HTML tagu od src do limit do dest */
{
  int i;
  char *p, *q;

#ifdef MIdprt
fprintf(stderr,"getcontextsuffix: bufflen: %i, maxlen_wanted: %i\n",limit-src,maxlen);
#endif
  
  /* predpoklada se, ze nejsme uprostred tagu */
  /* dalo by se to jeste osetrit flagem, ze pokud narazi driv na konec tagu
     nez na zacatek, tak se cukne zpet */
  for (i=0, p=src, q=dest;
       (limit-p>0) && i<maxlen;
       i++, p++, q++)
   {
    if (*p != '<')
      *q=( *p=='\n' || *p=='\r' ? ' ' : *p );
      /* netag kopiruj, rus konce radku prepisovanim na mezeru */
    else
     {
      /* preskoc tag */
      while ( *p != '>'  &&  limit-p>0)
        p++;
      if (limit-p == 0)
       /* jsme na konci */
        break;
      *q=' '; /* tag preskoc, dej misto nej bile misto */
     }
   }
  *q=0; /* ukoncit kopii */
  
#ifdef MIdprt
fprintf(stderr,"gotsuffix: >>>%s<<<\n",dest);
#endif
}

void
get_context_prefix(char *dest, char *src, char *limit, int maxlen)
/* kopiruje text mimo HTML tagu od src COUVANIM az do limit do dest */
{
  int i;
  char *p, *q;
  
#ifdef MIdprt
fprintf(stderr,"getcontextprefix: bufflen: %i, maxlen_wanted: %i\n",src-limit,maxlen);
#endif
  /* predpoklada se, ze nejsme uprostred tagu */
  /* dalo by se to jeste osetrit flagem, ze pokud narazi driv na konec tagu
     nez na zacatek, tak se cukne zpet */
  for (i=0, p=src, q=dest;
       (p-limit>0) && i<maxlen;
       i++, p--, q++)
   {
    if (*p != '>')
      *q=( *p=='\n' || *p=='\r' ? ' ' : *p );
      /* netag kopiruj, rus konce radku prepisovanim na mezeru */
    else
     {
      /* preskoc tag */
      while ( *p != '<'  &&  p-limit>0)
        p--;
      if (limit-p == 0)
       /* jsme na konci */
        break;
      *q=' '; /* tag preskoc, dej misto nej bile misto */
     }
   }
  *q=0; /* ukoncit kopii */
  strrev(dest); /* cele jsme to delali pozpatku! */
#ifdef MIdprt
fprintf(stderr,"gotprefix: >>>%s<<<\n",dest);
#endif
}

void
remember_context(int found, file_info *finf,
                 char* buff, char *b, int i, int bufflen)
{
  int samplen, filled;
  int e; /*extra_bytes;  jak siroke okoli nalezeneho vzorku bychom chteli*/
  
  finf->context=(char *)malloc(CONTEXT_LEN+1);
  if (finf->context == NULL)
    return;
    /* neni dost pameti na zapamatovani kontextu, kasli na to,
       ale zkus to priste */
       
  samplen=strlen(get_sample(found));
  e=(CONTEXT_LEN - samplen)/2;
  
#ifdef MIdprt
fprintf(stderr,"remembercontext:
bufflen: %i
found_pos: %i
extend_wanted: %i
",bufflen,i,e);
#endif

  get_context_prefix(finf->context, b-samplen-1, buff, e);
  /* zde mozno strcat(finf->context, "<B>"); */
  strcat(finf->context, get_sample(found));
  /* zde mozno strcat(finf->context, "</B>"); */
  filled = strlen(finf->context);
  get_context_suffix(finf->context + filled, b, buff+bufflen,
                     CONTEXT_LEN - filled);

#ifdef MIdprt
fprintf(stderr,"GOTCONTEXT: >>>%s<<<\n",finf->context);
#endif
  
  return;
}




/* --- More Info for file - jak udelat, aby jsme k souboru znali i jeho title a description --- */
/* Falesna id vzorku pro get_more_info, masinka nam pri hledani bude tohle plivat */
#define MI_TAG_START    -300
#define MI_TAG_END      -301
#define MI_META         -400
#define MI_DESCRIPTION  -401
#define MI_TITLE        -500
void
add_MI_samples(machine *m, int (*comp)(letter, letter))
/* pridej k automatu >m< specialni vzorky, ktere nam pomohou
   najit TITLE a DESCRIPTION */
{
  add_sample(m, MI_TAG_START,   "<",           comp);
  add_sample(m, MI_TAG_END,     ">",           comp);
  add_sample(m, MI_META,        "META",        comp);
  add_sample(m, MI_DESCRIPTION, "DESCRIPTION", comp);
  add_sample(m, MI_TITLE,       "TITLE",       comp);
}

int
get_more_file_info(int found, file_info *finf, char *buff,
      char *b, int i, int bufflen)
/* found je id MI_vzorku, ktery byl prave nalezen, get_more_file_info na to ma zareagovat
   pripadnym vycucnutim pozadovane informace */
#define WH      "%[ \n\r\t]"
#define DESC    "%[^\"]"
{
  static int in_meta;   /* tady si g_m_f_i pamatuje, v jakem je zrovna stavu */
  static int in_tag;    /* tj. co videl v nedavne minulosti */
  char *buff_end = buff + bufflen;
  char *r;
  char c;
  char wh[60];
  int titlelen;

  if (buff==NULL)
    /* inicializacni volani pred kazdym novym souborem, nastav si staticy */
   {
    in_meta=0;
    in_tag=0;
    return 0;
   }
  switch (found)
  {
    case MI_TAG_START:    /*  nalezeno <  */
      in_tag=1;
      break;
    case MI_TAG_END:      /*           >  */
      in_meta=0;
      in_tag=0;
      break;
    case MI_META:         /*         META */
#ifdef MIdprt
fprintf(stderr,"Narazil na META, intag: %i.\n",in_tag);
#endif
      if (in_tag)
        in_meta=1;
      break;
    case MI_DESCRIPTION:  /*  DESCRIPTION */
      if (!in_meta || finf->description)
        /* je to plany poplach nebo uz description nacucan mame */
        break;
#ifdef MIdprt
fprintf(stderr,"Narazil na DESC.\n");
#endif
      b=strchr(b,'"');
      if (!b)
        /* chybi tam rovnitko !! FIXME, mozna jsme uz mimo tag */
        break;
      finf->description=(char*)malloc(DESCRIPTION_LEN);
      if (! finf->description)
        /* neni dost pameti, kasli na to */
        break;
#ifdef MIdprt
fprintf(stderr,"Sscanf na DESCR.\n");
#endif
      *(finf->description)=0;
      sscanf(b, "\"" DESC "\"",
                finf->description);
      if (*(finf->description) == 0)
        /* nepodarilo se mu precist ten popis */
       {
        free(finf->description);
        finf->description = NULL;
        break;
       }
       untag_str(finf->description);
       strchrrepl(finf->description, '\n' ,' ');
       strchrrepl(finf->description, '\r', ' ');

#ifdef MIdprt
fprintf(stderr,"Extrahoval DESCR: >>>%s<<<\n",finf->description);
#endif
       /* interested = (finf->title != NULL);   tohle je nejaka blbost, to sem asi nepatri */
       break;
    case MI_TITLE:        /*  TITLE */
       /* jsme ve stavu ...TITLE~~b~~...>ZAJIMAVY_TITULEK</TITLE> */
#ifdef MIdprt
fprintf(stderr,"Narazil na TITLE.\n");
#endif
       if (! in_tag || finf->title)
         /* plany poplach nebo uz title mame */
         break;
       b = strchr(b, '>');
       if (b == NULL || buff_end - b < 0)
         /* nenasel jsem ani konec tagu <TITLE> */
         break;
       b++;
       r = strchr(b, '<');
       if (r == NULL || buff_end - r < 0)
         /* jako title vezmi, co se jeste veslo do bufferu, zbytek zapomen */
         r = buff_end;
       
       titlelen=r-b;
       if (titlelen > TITLE_LEN)
         /* pretekl povolenou velikost, nucene ho zkratme */
         titlelen=TITLE_LEN;
       finf->title = (char*)malloc(titlelen+1);
       if (finf->title)
        {
         /* bylo dost pameti, zkopirujme to; jinak bychom titulek klidne zapomneli */
         strncpy(finf->title, b, titlelen);
         *(finf->title + titlelen) = 0; /* ukoncovani 0 */
         strchrrepl(finf->title, '\n' ,' ');
         strchrrepl(finf->title, '\r', ' ');
        }
#ifdef MIdprt
fprintf(stderr,"Extrahoval TITLE: >>>%s<<<\n",finf->title);
#endif
       break;
  }
  return (!finf->title || !finf->description);
}


void
search_buffer(machine *m, char* buff, size_t bufflen, file_info *finf)
{
  char *b;
  int found, i;
  int interested;
  interested=(!finf->title || !finf->description);
   /* bude-li zajem cucat ze souboru TITLE a DESCRIPTION */
  for (b=buff, i=0;
       i<bufflen;
       b++, i++)
   {
    found=scan_machine(m, *b, (&ltr_cmpi) );
    while (  found != NOTHING_NEW  )
     {
      if (found>=0)
       {
        if (finf->context == NULL)
          /* potrebujeme si zapamatovat kontext */
{
#ifdef MIdprt
fprintf(stderr,"up_to_call remember_context:
bufflen: %i
found_offset %i
calced f_off %i
", bufflen, i, b-buff);
#endif
          remember_context(found, finf, buff, b, i, bufflen);
}
        count_sample(found);
        /* zapocti vzorek */
       }
      else
        /* je to ridici "vzorek", ktery nam pomaha najit TITLE, DESCRIPTION */
        if (interested)
          interested=get_more_file_info(found, finf, buff, b, i, bufflen);
      found=scan_machine(m,*b,(&ltr_cmpi));
     }
   }
}

int
decide_charset(int fhan, char* buff, int bufflen)
/* z bufferu poznej, je-li v souboru META s udanim charsetu */
{
  char *p;
  char *buff_end = buff+bufflen;
#define WH      "%[ \n\r\t]"
#define TAG     "%[^ \n\r\t>]"
#define ARG     "%[^ \r\n\t=>]"
#define VALUE   "%[^\"]"
#define HEADER  "%[^ \t:]"
#define DATA    "%[\x01-\xff]"
#define TYP     "%[^ \t;]"

  char tag[60];
  char arg[180];
  char val[240];
  char wh[60];
  char header[100];
  char data[140];
  char typ[60];
  char charset[80];
  int i;
  if (! config_int(C_CHARSET_META_CAN_OVERRIDE) )
    return config_int(C_CHARSET_SAVED_DEFAULT);

  p=buff;
  while ((buff_end-p) > 0 )
   {
     *tag=0;*arg=0;
     *val=0;*wh=0;
     *header=0;*data=0;
     *typ=0;*charset=0;
     p=(char*)memchr(p,'<',buff_end-p);
     if (p==0)
       return config_int(C_CHARSET_SAVED_DEFAULT);
     i=sscanf(p,"<" WH TAG WH ARG WH "=" WH "\"" VALUE "\"",
                    wh,tag,wh,arg,wh,    wh,     val );
     p++;
     if (*val == 0) continue;
     if ( 0 != strcmpi(tag,"META") ) continue;
     if ( 0 != strcmpi(arg,"HTTP-EQUIV") ) continue;
     i=sscanf(val, WH HEADER WH ":" WH DATA,
                   wh,header,wh,    wh,data);
     if (*data == 0) continue;
     if ( 0 != strcmpi(header,"Content-type") ) continue;
     i=sscanf(data, "text/" TYP ";"  WH "charset" WH "=" WH "%s",
                            typ,     wh,          wh,    wh, charset);
     if (*charset == 0)
     /* nasli jsme meta tag, ale neni v nem popis charsetu, vzdejme to */
       return config_int(C_CHARSET_SAVED_DEFAULT);
     i=a_to_charset(charset);
     if (i == CH_UNKNOWN)
       return config_int(C_CHARSET_SAVED_DEFAULT);
     else
       return i;
   }
  return config_int(C_CHARSET_SAVED_DEFAULT);
  /* fhan se mozna bude hodit pozdeji, kdyby to bralo v potaz .htaccess
     nebo tak */
}



/* ----------- souborove operace ---------- */
/* cteni souboru je postaveno na mapovani do pameti */

#include <fcntl.h>

#ifdef dos
#include <io.h>
#define O_BINARY 0
#endif

#include <sys/mman.h>



int
init_file_info(file_info *finf)
{
  finf->charset = CH_UNKNOWN;
  finf->title = NULL;
  finf->context = NULL;
  finf->description = NULL;
  /* finf->file_stat    neinicializovano!!! */
  return 0;
}




int
search_file(char *pathname, file_info *finf)
/* prohleda soubor na vzorky postavenou masinkou
   poscita vyskyty do SAMPLE(id).count  */
 {
  char *buff1;   /* sem se namapuje */
  int buff1_len;
  int desired_file_charset;
  char *buff2;   /* tady se kdyztak prevede znakova sada */
  int buff2_len;
  char *scanbuff; /* to je odkaz na buffer, ktery se fakt ma prohledat */
  int scanbufflen; /* cili buff1 nebo buff2, podle toho, zda se prevadelo */
  
  int fh;
  int offset;
    /* co uz jsme precetli - nevyuziva se, protoze mapujeme vsechno naraz */
   
  struct conv_context prevod;
  machine *m; /* bude odkaz na stroj, kterym se fakticky bude prohledavat */
  
  
  get_more_file_info(0,0,0,0,0,0);
     /* inicializacni volani */
  init_file_info(finf);
  
#ifdef searprt
fprintf(stderr,"Searcher: Otviram k prohledani >>%s<<\n",pathname);
#endif
  fh=open(pathname,O_RDONLY);
  if (fh==-1)
    return OPEN_FAILED;
  if ( fstat(fh, &(finf->file_stat)) != 0)
    return GET_INFO_FAILED;
  
  /* -- namapovani -- */
  buff1_len=finf->file_stat.st_size; /* namapujme cely soubor naraz */
  offset=0;
  buff1 = mmap(NULL, buff1_len, PROT_READ,
#ifdef dec              
              MAP_FILE  |  MAP_VARIABLE  |  
#endif
               MAP_PRIVATE,
              fh, offset);
  if (! buff1)
    return OPEN_FAILED;
    
#ifndef linux  
  madvise(buff1, buff1_len, MADV_SEQUENTIAL);
  /* jako ze to budeme delat tak */
#endif

  /* -- mame namapovano v buff1 o delce buff1_len -- */
  
  /* -- prohledani buff1 -- */
  /* Napred se zjisti znakova sada souboru. Kdyz jsou soubor i vzorky v
  dia, jedu diamach, ale musim zajistit, aby byl soubor v INTERNALU. Kdyz
  cokoli z toho je ascii, jedu ascii, cili musim zajistit, aby i soubor
  byl ascii */

  if (finf->charset == CH_UNKNOWN)
   {
#ifdef searprt
fprintf(stderr,"Searcher: U souboru zkusim poznat znakovou sadu decide_charsetem\n");
#endif
    finf->charset = decide_charset(fh, buff1, buff1_len);
   }

  if (config_int(C_CHARSET_NETWORK_INPUT) != CH_ASC
       && finf->charset != CH_ASC )
   {
#ifdef searprt
fprintf(stderr,"Searcher: Soubor i vzorky jsou v dia, jedu dia.\n");
#endif
    /* vzorky jsou v dia, pojedu diamach, a soubor musi byt v INTERNAL!
    */
    m=dia_mach;
    desired_file_charset=INTERNAL_CHARSET;
   }
  else
   {
#ifdef searprt
fprintf(stderr,"Searcher: Soubor nebo vzorky v ASCII, jedu ascmach.\n");
#endif
    /* vzorky mam v ascii, budu urcite prohledavat v ascii */
    m=asc_mach;
    desired_file_charset=CH_ASC;
   }

  reset_machine(m);

  if (finf->charset == desired_file_charset )
   {
#ifdef searprt
fprintf(stderr,"Searcher: Ulozeny soubor je v pozadovane znakove sade, neprevadim.\n");
#endif
    buff2 = NULL; /* nic nealokuj */
    scanbuff=buff1; /* uzij neprevadene */
    scanbufflen=buff1_len;
   }
  else
   {
#ifdef searprt
fprintf(stderr,"Searcher: Ulozeny soubor prevedu %s -> %s.\n", charset_to_a(finf->charset,charset_name_MIME), charset_to_a(desired_file_charset,charset_name_MIME));
#endif
    buff2 = (char*)malloc(sizeof(char)*(buff1_len+1)); /* +1 je rezerva */
    if (!buff2)
      goto memfail;
    buff2_len = buff1_len;
    charset_convert_init(&prevod, buff1, buff1+buff1_len,
                                  buff2, buff2+buff2_len);
    charset_convert(&prevod, finf->charset, desired_file_charset);
    /* FIXME preteceni jednoho ci druheho bufferu */
    scanbuff=buff2;
    scanbufflen=buff1_len;
   }

  search_buffer(m,scanbuff,scanbufflen,finf);
  if (buff2 != NULL)
    free(buff2);
  /* ted je cely buffer prohledan */
  
  munmap(buff1, buff1_len);
  close(fh);
  return 0;
  
  
  memfail:
  munmap(buff1, buff1_len);
  close(fh);
  return OUT_OF_MEMORY;
};

void
search_files(FILE *input,FILE *output)
// prohledava vsechny soubory, jejichz jmena jsou na stdin a vystup sype na output
{
  char pathname[PATH_NAME_LEN];
  file_info finf;
  int i;

#ifdef dbgprt
fprintf(stderr,"searcher: up to search files\n");
#endif

  while ( my_fgets(pathname,PATH_NAME_LEN,input) != NULL )
   {
#ifdef dbgprt
fprintf(stderr,"searcher: scanning [%s]\n",pathname);
#endif
    if (strcmp(pathname,ALL_FILES_FINISHED) == 0)
      break;
    
    // vynulovat pocitadla vzorkum
    reset_samples_counter();
    search_file(pathname,&finf);
    flush_results(output,&finf,pathname);
#ifdef dbgprt
fprintf(stderr,"Searcher: search_files jde na dalsi soubor.\n");
#endif

    // vyplivej vysledky
   };
  printf(ALL_FILES_FINISHED "\n");
  /* plivni i oddelovac */
};




static config_info searcher_config_info [] =
{
  {  C_CHARSET,      C_CHARSET_SAVED_DEFAULT,        C_KEEP },
  {  C_BOOL,         C_CHARSET_META_CAN_OVERRIDE,    C_KEEP },
  {  C_CHARSET,      C_CHARSET_NETWORK_OUTPUT,       C_PASS },
  {  C_CHARSET,      C_CHARSET_NETWORK_INPUT,        C_KEEP },
//  {  C_BOOL,         C_CHARSET_ADD_TO_CONTENT_TYPE,  C_PASS },
//  {  C_STRING,       C_ROOT_DIR,                     C_PASS },
//  {  C_STRING,       C_DIR_DELIMITER,                C_PASS },
//  {  C_STRING,       C_ROOT_URL,                     C_PASS },
//  {  C_STRING,       C_YAAHO_CGI_URL,                C_PASS },
//  {  C_BOOL,         C_HTTP_METHOD_POST,             C_PASS },
  {  C_INT,          C_REQUEST_OFFSET,               C_PASS },
  {  C_INT,          C_REQUEST_COUNT_LIMIT,          C_PASS },
//  {  C_INCLUDE,      C_LAYOUT_ERROR_PAGE,            C_KEEP },
//  {  C_INCLUDE,      C_LAYOUT_ANSWER_PAGE_HEADER,    C_KEEP },
//  {  C_INCLUDE,      C_LAYOUT_ANSWER_PAGE_ITEM,      C_KEEP },
//  {  C_INCLUDE,      C_LAYOUT_ANSWER_PAGE_FOOTER,    C_KEEP },
  {  C_STRING,       C_PLOTTER_PATHNAME,             C_KEEP },
//  {  C_STRING,       C_SEARCHER_PATHNAME,            C_PASS },
//  {  C_STRING,       C_FIND_COMMAND,                 C_PASS },
  {  C_END,          C_NO_CID,                       C_PASS }
};

/* --- inicializace a priprava */
void
construct_blank_machines()
/* postavi jeden az dva prazdne strojecky */
{
  /* ascii masinku budeme potrebovat vzdycky */
  asc_mach = (machine *)malloc(sizeof(machine));
  if (asc_mach==NULL)
    fatal(OUT_OF_MEMORY);
  init_machine(asc_mach);
  add_MI_samples(asc_mach, ltr_cmpi);

  /* dia masinku jenom pokud na vstupu byla diakritika */
  if (config_int(C_CHARSET_NETWORK_INPUT) == CH_ASC)
    /* bude nam stacit jedna masinka */
    dia_mach = NULL;
  else
   {
    dia_mach = (machine*)malloc(sizeof(machine));
    if (dia_mach == NULL)
      fatal(OUT_OF_MEMORY);
    init_machine(dia_mach);
    add_MI_samples(dia_mach, ltr_cmpi);
   }
}

void finish_machines()
{
  finish_machine(asc_mach,ltr_cmpi);
  if (config_int(C_CHARSET_NETWORK_INPUT) != CH_ASC)
    finish_machine(dia_mach,ltr_cmpi);
    /* jsou to odlisne strojky dodelat zvlast*/
  return;
}


#ifdef dbgprt
#include <errno.h>
#endif

#ifdef PIPED

int
run_and_link_plotter()
/* pripoji plotter spusti plotter a pripoji stdout na jeho stdin */
{
  int err;

#ifdef dbgprt
fprintf(stderr,"Searcher: Spoustim plotter >>%s<<.\n",config_str(C_PLOTTER_PATHNAME));
#endif
  
  err=pipe_to_fork(config_str(C_PLOTTER_PATHNAME));
  
  if (err < 0)
  /* err je -1 pokud se nepovedl ani fork, -2, pokud se fork povedl
        (pak tu chybu dostane synacek, ocet si bude myslet, ze je to ok) */
   {
#ifdef dbgprt
fprintf(stderr, "Nepovedlo se spustit plotter, err=%i, errno=%i.\n",err, errno);
#endif
    return PLOTTER_EXEC_FAILED;
   }
  
#ifdef dbgprt
fprintf(stderr, "Searcher: Navazal jsem na sebe plotter, plotter_pid=%i.\n",err);
#endif
  
  return 0;
}
#endif /* PIPED */




/* --- main */

int
main(int argc,char *argv[])
{
  char *fname;
  int err;
  FILE *output;
  FILE *input;
  input = stdin;
  
#ifdef dbgprt
fprintf(stderr,"Searcher entry point.\n");
#endif

#ifdef PIPED
  prefatal_state = not_linked;
#else
  prefatal_state = nothing_written;
#endif
  output = stdout;
  
  prefatal_state = nothing_written;
  fname=get_config_file_name(argc,argv);
  if (fname==NULL)
    fname=DEFAULT_CONFIG_FILE;

  err=init_config(searcher_config_info);
  if (err)
    fatal(err);
  err=read_config_from_file/*_and_stdin*/(fname);
  if (err)
    fatal(err);

  err=read_config(input); /* nacti jeste ze stdin */
  if (err)
    fatal(err);

  /* mame nactenou konfiguraci, tedy muzeme spustit plotter */
#ifdef PIPED
  if (err=run_and_link_plotter())
    fatal(err);
#endif

  init_samples();
  construct_blank_machines();
  read_samples(input);

  finish_machines();

  flush_headers(output);

#ifdef dbgprt
fprintf(stderr,"Searcher ready to scan files.\n");
#endif

  search_files(input,output);
  prefatal_state = files_written;

#ifdef dbgprt
fprintf(stderr,"Searcher checking input for fatal.\n");
#endif
  check_input_for_fatal(input);
  /* co kdyby tam visela chyba od yaaho */

  /* masinky a jine struktury NIKDO NERUSI, FIXME */

#ifdef dbgprt
fprintf(stderr,"Searcher quitting...\n");
#endif
  return 0;
}

