/*
  yaaho.c
  obo@cuni.cz
  Entry point to the yaaho services.
*/

#define DEFAULT_CONFIG_FILE "yaaho.conf"

#include "cross.h"
#include "config.h"
#include "charset.h"
#include "samples.h"
#include <stdlib.h>
#include <string.h>
#include "strtools.h"
#include "myexec.h"

#ifdef myfgetsdebug
char *myfgets_id="yaaho";
#endif

static config_info yaaho_config_info [] =
{
//  {  C_CHARSET,      C_CHARSET_SAVED_DEFAULT,        C_PASS },
//  {  C_BOOL,         C_CHARSET_META_CAN_OVERRIDE,    C_PASS },
  {  C_CHARSET,      C_CHARSET_NETWORK_OUTPUT,       C_PASS },
  {  C_CHARSET,      C_CHARSET_NETWORK_INPUT,        C_PASS },
     /* searcher se musi dozvedet, jestli jedeme ascmach nebo diamach */
//  {  C_BOOL,         C_CHARSET_ADD_TO_CONTENT_TYPE,  C_PASS },
  {  C_BOOL,         C_7BIT_ASSUME_ASCII,            C_KEEP },
//  {  C_STRING,       C_ROOT_DIR,                     C_PASS },
//  {  C_STRING,       C_DIR_DELIMITER,                C_PASS },
//  {  C_STRING,       C_ROOT_URL,                     C_PASS },
//  {  C_STRING,       C_YAAHO_CGI_URL,                C_PASS },
  {  C_BOOL,         C_HTTP_METHOD_POST,             C_KEEP },
  {  C_INT,          C_REQUEST_OFFSET,               C_PASS },
  {  C_INT,          C_REQUEST_COUNT_LIMIT,          C_PASS },
//  {  C_INCLUDE,      C_LAYOUT_ERROR_PAGE,            C_KEEP },
//  {  C_INCLUDE,      C_LAYOUT_ANSWER_PAGE_HEADER,    C_KEEP },
//  {  C_INCLUDE,      C_LAYOUT_ANSWER_PAGE_ITEM,      C_KEEP },
//  {  C_INCLUDE,      C_LAYOUT_ANSWER_PAGE_FOOTER,    C_KEEP },
  {  C_STRING,       C_PLOTTER_PATHNAME,             C_PASS },
  {  C_STRING,       C_SEARCHER_PATHNAME,            C_KEEP },
  {  C_STRING,       C_FIND_COMMAND,                 C_KEEP },
  {  C_END,          C_NO_CID,                       C_PASS }
};


void help_and_quit()
{
  printf(
  "yaaho\n"
  "http://www.cuni.cz/~obo/skola/yaaho/\n"
  "Entry point to the yaaho services./n"
  "\n"
  "Yet-Another-AHO_corasic_based search machine\n"
  "\n" );
  exit(0);
}
void fatal(int kind)
{
  if (fatal_flush(stdout,kind))
    ;
  
  /* pokud jsme se dostali sem, mame za ukol hlasit chybu sami */
  
  printf("Yaaho: %s\n%i\n",error_string(kind),kind);
  exit(2);
};

/* ----- zpracovani HTTP hlavicek ----- */
void
resolve_accept_charset(char *str)
/* str je typu   iso-8859-2; q = 1, windows-1250; q=0.003  */
{
  int best_chst  = CH_UNKNOWN;
  int chst       = CH_UNKNOWN;
  float  best_q  = 0;
  float  q       = 0;
  char *itm, *quo;
  for ( itm = strtok(str, ";,")  ;
        itm ;
        itm = strtok(NULL,";,")  )
   {
    itm=skipwhite(itm);
    rtrimwhite(itm);
    quo=strchr(itm,'=');
    if (quo && *quo)
     {
      /* aktualni itm je nejspis kvocient */
      *quo=0; quo++;
      /* itm by melo byt "q", quo by melo byt cislo, itm je jmeno
        toho charsetu, udelejme z nej cislo */
      if (0 == strcmpiw(itm, "q"))
       {
        /* je to opravdu kvocient */
        sscanf(quo, "%f", &q);
        if ( chst != CH_UNKNOWN && q>best_q )
         {
          /* chst je posledni nacteny charset, ten pred nactenym kvocientem, nyni
             zvitezil v boji nad dosud nejlepsim, zapamatujme si jej */
          best_chst=chst;
          best_q=q;
         }
        chst = CH_UNKNOWN;
        continue; /* spagety, potreboval jsem proste spolecny else pro dva vnorene ify */
       }
      /* ted vime, ze itm neni kvocient, musime jej zrekonstruovat a pak normalne
        parsovat */
      quo--;
      *quo = '=';
     }
    if (chst != CH_UNKNOWN)
     /* sem jsme se dostali, kdyz jdou dva neqovane charsety hned za sebou
        v tomto pripade tedy plati pro prvni z nich (dosud v chst) q=1
        a takovym musime dosud best_chst pretlouct, k tomu dojde jakmile
        opustime cyklus */
      break;
    chst = a_to_charset(itm);

    /* itm je jmeno charsetu, potrebujeme si ho zapamatovat */
    if (best_chst == CH_UNKNOWN)
      best_chst = chst;
   } /*konec cyklu pres itm */
  if ( chst != CH_UNKNOWN )
   {
    /* chst je posledni nacteny charset a nemel ani za sebou omezujici q,
      takze plati!! */
    best_chst=chst;
   }

  if (best_chst != CH_UNKNOWN)
    set_config_int(C_CHARSET_NETWORK_OUTPUT, best_chst);
}
void
load_config_from_http_headers()
{
  char *value;
  
  value = getenv("HTTP_METHOD");
  if (0 == strcmpi(value,"POST"))
    set_config_int(C_HTTP_METHOD_POST,1);
  /* else neni treba, protoze konfig je autoinicilizovan */
  
  value = getenv("ACCEPT_CHARSET");
  if (value)
    resolve_accept_charset(value);
  
}

/* ----- zpracovani query ----- */

#define QUERY_LEN       10240
#define QUERY_ITEM_LEN   2048

#define INPUT_CHARSET_TAG "input_charset"
#define VIEW_CHARSET_TAG  "view_charset"
#define SAMPLES_TAG       "samples"

void
resolve_charset_in_advance(char *buff)
/* ukolem je z QUERY vyseknout charset_network_input uplne hned na zacatku,
   resp. dost brzo, aby se nestalo, ze napred prijdou vzorky a pak teprve bude
   nastaven tento konfig */
{
  char *p, *q;
  int chst;
  
  p = strstr(buff, "&" INPUT_CHARSET_TAG "=");
//printf("resolve_charset_in_advance: %s",p);
  if (!p)
    return;
  /* muzeme si dovolit nutne vyzadovat ten &, protoze je-li charset uveden jako prvni,
    tj. bez &, jako prvni ho regulerne nacteme a popsany problem nenastane */
  p+=strlen("&" INPUT_CHARSET_TAG "="); /* tj. delka &charset= */
  q=strchr(p,'&');
  if (q) *q=0;
  chst=a_to_charset(p);
  if (q) *q='&';
  if (chst != CH_UNKNOWN)
    set_config_int(C_CHARSET_NETWORK_INPUT, chst);
}

void
load_samples(char *buff)
/* nacti vzorky typu
     +sysel "jouda je vul" +"petr taky"\0   */
{
  char *buff2;
  int buff2_len;
  char *p, *q;
  int required;
  int sevenbitonly;
  char *sbittest;

  sevenbitonly = 1;

  if (  config_int(C_CHARSET_NETWORK_INPUT) != INTERNAL_CHARSET   )
   {
    /* musime vzorky prevest do interniho kodovani */
    buff2_len = strlen(buff);
    buff2 = (char*)malloc(buff2_len+1);
    if (! buff2) fatal(OUT_OF_MEMORY);
    charset_convert_string(buff, buff2, buff2+buff2_len+1,
       config_int(C_CHARSET_NETWORK_INPUT), INTERNAL_CHARSET  );
//printf("samples: [%s]@%i->[%s]@%i\n",buff,
//config_int(C_CHARSET_NETWORK_INPUT),buff2, INTERNAL_CHARSET);
   }
  else
   {
    buff2=buff;
    buff2_len = -1; /* znak, ze je nealokovano */
   }
//printf("samples to read: [%s]\n",buff2);
  p = buff2;
  p=skipwhite(p);
  while (*p)
   {
    required=0;
    if ( *p == '+' )
     {
      required = 1;
      p++;
      if ( ! *p ) break;
     }
    if ( *p == '"' )
     {
      p++;
      q = strchr(p, '"');
      if (q) *q=0;
     }
    else
     {
      q = p + strcspn(p, WHITE_SET);
      if (q && *q==0) q=NULL;
      if (q) *q=0;
     }

    remember_sample(p, required);
    if (sevenbitonly)
      for (sbittest = p; *sbittest; sbittest++)
        if (((unsigned char)*sbittest) > 127)
	 {
	  sevenbitonly=0;
	  break;
	 }

    if (q)
      p=q+1;
    else
      break; /* vime, ze koncime */
    
    p = skipwhite(p);
   } /* while */
  
  if (sevenbitonly && config_int(C_7BIT_ASSUME_ASCII))
    set_config_int(C_CHARSET_NETWORK_INPUT, CH_ASC);
  /* take by se dalo testovat !sevenbitonly && network input!=ascii
     a hlasit to jako problem, ale budme tolerantni */

} /* load_samples */



void
parse_query_pair(char *name, char *value)
{
  int chst;
  
  if (0 == strcmpiw(name,SAMPLES_TAG))
    load_samples(value);
  else if (0 == strcmpiw(name, INPUT_CHARSET_TAG))
   {
    chst=a_to_charset(value);
#ifdef searprt
fprintf(stderr,"input_charset: \"%s\", tj. %i=%s\n",value,chst,charset_to_a(chst,charset_name_MIME));
#endif
    if (chst != CH_UNKNOWN)
      set_config_int(C_CHARSET_NETWORK_INPUT, chst);
   }
  else if (0 == strcmpiw(name, VIEW_CHARSET_TAG))
   {
    chst=a_to_charset(value);
#ifdef searprt
fprintf(stderr,"output_charset: \"%s\", tj. %i=%s\n",value,chst,charset_to_a(chst,charset_name_MIME));
#endif
    if (chst != CH_UNKNOWN)
      set_config_int(C_CHARSET_NETWORK_OUTPUT, chst);
   }
  else ; /* dalsi polozky ignoruj */
}

void
parse_query_buffer(char *buff)
{
  char name[QUERY_ITEM_LEN];
  char value[QUERY_ITEM_LEN];
  char *itm;
  
  resolve_charset_in_advance(buff);
  
  for ( itm=strtok(buff,"&"); itm; itm=strtok(NULL,"&") )
   {
    quextr(itm, name, value);
    parse_query_pair(name, value);
   }
}

void
parse_query()
{
  char buffer[QUERY_LEN];
  char *buff;
  
  if (config_int(C_HTTP_METHOD_POST))
    buff = gets(buffer);
  else
    buff = getenv("QUERY_STRING");
    
  if (buff == NULL)
    fatal(QUERY_NOT_FOUND);
  
  parse_query_buffer(buff);
}

/* ----- posledni kroky pred spustenim neceho dalsiho ----- */
int
ready_to_run()
/* testuje minimalni pozadavky pred spustenim */
{
  if (samples_count() == 0)
    return NO_SAMPLES_SPECIFIED;
  
  if (config_int(C_CHARSET_NETWORK_OUTPUT) == CH_UNKNOWN)
    set_config_int(C_CHARSET_NETWORK_OUTPUT, CH_ASC);
    /* nebo se rovnou sesyp na chybu, ze nevis, v cem to mas odeslat */
  
  return 0; /* muzeme pokracovat */
}

void
flush_files(FILE *output)
/* tj. spust find command */
{
  int err;
  
  /* musime vyplivnout dosavadni vystup */
  fflush(stdout);
  
#ifdef dbgprt
  fprintf(stderr, "...no a tady zavolam %s...\n", config_str(C_FIND_COMMAND));
#endif
#ifdef dbgprt
fprintf(stderr,"Yaaho timto konci, nahrazen findem, predtim oznaci CLOEXEC
na stdout.");
#endif
  
  if (0!=my_execp(config_str(C_FIND_COMMAND)))
    fatal(FIND_EXEC_FAILED);

  /* k tomuhle uz sice v dobrem pripade nedojde, ale to searcheru neva */
  fprintf(output,ALL_FILES_FINISHED "\n");
}


#include <errno.h>


int
exec_and_link_searcher()
{
  int sonpid;
  
  sonpid=pipe_to_fork(config_str(C_SEARCHER_PATHNAME));
  
  switch (sonpid)
   {
    case -1:
      /* neuspel, a jsme v otci, pred forkem */
    case -2:
      /* neuspel, ale uz jsme synacek */
      /* ono je to v podstate, jedno, musi jit chyba na vystup */
#ifdef dbgprt
      fprintf(stderr,"Spusteni searcheru se nepovedlo, errno=%i\n",errno);
#endif dbgprt
      return SEARCHER_EXEC_FAILED;
      break;

    default:
      /* sonpid je pid syna, je to uspech */
      return 0;      
   }
}

int main(int argc, char *argv[])
{
  char *fname;
  int err;
  FILE *output;

#ifdef HTTP_DEBUG
  printf("Content-type: text/plain\n\n");
  fflush(stdout);
#endif
#ifdef PIPED
  prefatal_state = not_linked;
#else
  prefatal_state = nothing_written;
#endif

  err = init_config(yaaho_config_info);
  if (err) fatal(err);
  fname=get_config_file_name(argc, argv);
  if (fname == NULL)
    fname = DEFAULT_CONFIG_FILE;
  err = read_config_from_file(fname);
  if (err) fatal(err);

  init_samples();

  /* FIXME falesne prostredi! */
//  putenv("HTTP_METHOD=POST");
//  putenv("ACCEPT_CHARSET=us-ascii;q=0.01,windows-1250;q=0.1;iso-8859-2,sdfn");

  /* nacist zivou konfiguraci ze site, uhodnout i vstupni charset co nejdriv */
  load_config_from_http_headers();

  /* a nacist take vzorky k vyhledani */
  parse_query();

#ifdef HTTP_DEBUG
  fprintf(stderr,"Yaaho: tuhle konfiguraci posilam dal.\n");
  flush_config(stderr);
  fflush(stderr);
#endif

  err = ready_to_run();
  if (err) fatal (err);

  /* pripojit searcher!! */
#ifdef PIPED
  if ( err=exec_and_link_searcher() )
    fatal(err);
#endif
  output=stdout;

  prefatal_state = nothing_written;

  flush_config(output);
  prefatal_state = config_written;
  
  flush_samples_names(output);
  fprintf(output,SAMPLES_FINISHED "\n");
  prefatal_state = samples_written;

#ifdef PIPED
  flush_files(output);
#endif
  prefatal_state = files_written;

  return 0;
}
